<?php

declare(strict_types=1);

namespace Drupal\ga4_server_events\EventSubscriber;

use Drupal\commerce_price\NumberFormatter;
use Drupal\commerce_price\Price;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\state_machine\Event\WorkflowTransitionEvent;
use Psr\Http\Client\ClientInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * @todo Add description for this subscriber.
 */
final class Ga4ServerEventsSubscriber implements EventSubscriberInterface {

  /**
   * Constructs a Ga4ServerEventsSubscriber object.
   */
  public function __construct(
    private readonly ClientInterface $client,
    private readonly ConfigFactoryInterface $configFactory,
    private readonly ModuleHandlerInterface $moduleHandler,
    private readonly NumberFormatter $numberFormatter,
    private readonly LoggerChannelFactoryInterface $loggerFactory,
  ) {}

  /**
   * Responds to the "commerce_order.place.post_transition" event.
   *
   * @param \Drupal\state_machine\Event\WorkflowTransitionEvent $event
   *   The event object.
   */
  public function onOrderPlace(WorkflowTransitionEvent $event) {
    $config = $this->configFactory->get('ga4_server_events.settings');
    if ($config->get('disable')) {
      return;
    }
    $api_secret = $config->get('api_secret');
    $measurement_id = $config->get('measurement_id');
    if (!$api_secret || !$measurement_id) {
      return;
    }
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $event->getEntity();
    $client_id = ga4_server_events_get_client_id($order->getEmail());
    $url = "https://www.google-analytics.com/mp/collect?measurement_id={$measurement_id}&api_secret={$api_secret}";
    $currency_code = $order->getTotalPrice()?->getCurrencyCode() ?: $order->getStore()?->getDefaultCurrencyCode();

    $event_data = [
      'name' => 'purchase',
      'params' => [
        'transaction_id' => $order->getOrderNumber() ?: $order->id(),
        'currency' => $currency_code,
        'value' => $this->numberFormatter->format($order->getTotalPrice()?->getNumber(), ['use_grouping' => FALSE]),
        'engagement_time_msec' => 100,
      ],
    ];
    if (str_contains($client_id, '.')) {
      // We can add session_id (with engagement_time_msec), so purchase events
      // will appear in realtime and session reports.
      $event_data['params']['session_id'] = explode('.', $client_id)[1];
    }
    // Add information about coupons.
    if (!$order->get('coupons')->isEmpty()) {
      $event_data['params']['coupon'] = $order->get('coupons')->entity->getCode();
    }

    // Add information about shipping.
    if ($this->moduleHandler->moduleExists('commerce_shipping')) {
      $shipping_total = new Price('0', $currency_code);
      foreach ($order->collectAdjustments(['shipping']) as $adjustment) {
        $shipping_total->add($adjustment->getAmount());
      }
      $event_data['params']['shipping'] = $this->numberFormatter->format($shipping_total->getNumber(), ['use_grouping' => FALSE]);
    }

    // Add information about taxes.
    if ($this->moduleHandler->moduleExists('commerce_tax')) {
      $tax_total = new Price('0', $currency_code);
      foreach ($order->collectAdjustments(['tax']) as $adjustment) {
        $tax_total->add($adjustment->getAmount());
      }
      $event_data['params']['tax'] = $this->numberFormatter->format($tax_total->getNumber(), ['use_grouping' => FALSE]);
    }

    $data = [
      'client_id' => $client_id,
      'events' => [
        $event_data,
      ],
    ];
    if ($order->getCustomerId()) {
      // We can add user_id for extra identification.
      $data['user_id'] = $order->getCustomerId();
    }
    $this->moduleHandler->alter('ga4_server_events_data', $data);
    try {
      $this->client->post($url, ['form_params' => $data]);
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('ga4_server_events')->error('Failed to send GA4 event: @message', ['@message' => $e->getMessage()]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      'commerce_order.place.post_transition' => 'onOrderPlace',
    ];
  }

}
